﻿function HttpJsonRequestProvider() {

    //создаем объект, позволяющий делать HTTP-запросы
    var xmlhttp = new XMLHttpRequest();
    
    // url - ссылка на ресурс
    // sendData - данные для отправки
    // method - метод запроса ('GET', 'POST', 'PUT'...)
    //   Если method не был задан, то будет выполнен POST-запрос.
    // Поле url - обязательное.
    // Метод sendRequest возвращает результат запроса {status, response}.
    var sendRequest = function(url, sendData, method, authorizationToken) {
        if (!url)
            throw new Error('Url required.');
        //если метод не передан, то подразумевается POST
        method = method || "POST";
        //если sendData не строка, то делаем из нее или пустого объекта строку
        if (!(typeof sendData == 'string' || sendData instanceof String)) {
            sendData = JSON.stringify(sendData || {});
        }

        //создаем соединение
        xmlhttp.open(method, url, false);
        //задаем header
        xmlhttp.setRequestHeader("Accept", "json");
        xmlhttp.setRequestHeader("Content-Type", "application/json");
        //если приложен authorizationToken
        if (authorizationToken) {
            xmlhttp.setRequestHeader("ScoutAuthorization", authorizationToken);
        }
        //отправляем
        xmlhttp.send(sendData);

        //возвращаем статус и тело ответа
        var responseText = xmlhttp.responseText;
        var parsedJson;
        if (responseText) {
            try {
                //создаем объект из полученного JSON
                parsedJson = JSON.parse(responseText);
            } catch(e) {
                alert("Incorrect JSON response from server");
            }
        }
        return {
            status: xmlhttp.status,
            response: parsedJson
        };
    };

    // url - ссылка на ресурс
    // sendData - данные для отправки
    // method - метод запроса ('GET', 'POST', 'PUT'...)
    //   Если method не был задан, то будет выполнен POST-запрос.
    // callback - метод, вызывающийся при получении ответа от сервера. Возвращает responseText и status
    // Поле url - обязательное.
    var RequestAsync = function (url, sendData, method, authorizationToken, callback) {
        if (!url)
            throw new Error('Url required.');
        //если метод не передан, то подразумевается POST
        method = method || 'POST';
        //если sendData не строка, то делаем из нее или пустого объекта строку
        if (!(typeof sendData == 'string' || sendData instanceof String)) {
            sendData = JSON.stringify(sendData || {});
        }

        //создаем соединение
        xmlhttp.open(method, url, callback);
        //подключаем callback-функцию
        xmlhttp.onreadystatechange = function() {
            //Completed
            if (xmlhttp.readyState == 4) {
                var responseText = xmlhttp.responseText;
				var parsedJson;
                if (responseText) {
                    try {
                        parsedJson = JSON.parse(responseText);
                    } catch(e) {
                        alert("Incorrect JSON response from server");
                    }
                }
                callback(parsedJson, xmlhttp.status);
            }
        };
        //задаем header
        xmlhttp.setRequestHeader("Accept", "json");
        xmlhttp.setRequestHeader("Content-Type", "application/json");
        //если приложен authorizationToken
        if (authorizationToken) {
            xmlhttp.setRequestHeader("ScoutAuthorization", authorizationToken);
        }
        //отправляем
        xmlhttp.send(sendData);
    };

    this.GET = function (url, sendData) {
        return sendRequest(url, sendData, 'GET');
    };

    this.POST = function (url, sendData) {
        return sendRequest(url, sendData, 'POST');
    };

    this.GETAuthorized = function (url, sendData, authorizationToken) {
        return sendRequest(url, sendData, 'GET', authorizationToken);
    };

    this.POSTAuthorized = function (url, sendData, authorizationToken) {
        return sendRequest(url, sendData, 'POST', authorizationToken);
    };

    this.GETAsync = function (url, sendData, callback) {
        return RequestAsync(url, sendData, 'GET', null, callback);
    };

    this.POSTAsync = function (url, sendData, callback) {
        return RequestAsync(url, sendData, 'POST', null, callback);
    };

    this.GETAuthorizedAsync = function (url, sendData, authorizationToken, callback) {
        return RequestAsync(url, sendData, 'GET', authorizationToken, callback);
    };

    this.POSTAuthorizedAsync = function (url, sendData, authorizationToken, callback) {
        return RequestAsync(url, sendData, 'POST', authorizationToken, callback);
    };
}